@@ -142,9 +142,17 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 		 html_type_name (HTML_OBJECT_TYPE (obj)),
 		 obj->x, obj->y, obj->min_width, obj->width, obj->pref_width, obj->max_width, obj->ascent, obj->descent);
 
-	if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_CLUEFLOW)
+	if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_CLUEFLOW) {
 		g_print (" [%s, %d]",
 			 clueflow_style_to_string (HTML_CLUEFLOW (obj)->style), HTML_CLUEFLOW (obj)->levels->len);
+		g_print (" levels: [");
+		for (i = 0; i < HTML_CLUEFLOW (obj)->levels->len; i ++) {
+			g_print ("%d", HTML_CLUEFLOW (obj)->levels->data [i]);
+			if (i < HTML_CLUEFLOW (obj)->levels->len - 1)
+				g_print (" ");
+		}
+		g_print ("]");
+	}
 	else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) {
 		gchar *sl_text = g_strndup (html_text_get_text (HTML_TEXT (HTML_TEXT_SLAVE (obj)->owner),
 								HTML_TEXT_SLAVE (obj)->posStart),
@@ -250,9 +258,24 @@ dump_object_simple (HTMLObject *obj,
 		gtk_html_debug_list_links (text);
 		gtk_html_debug_list_text_attrs (text);
 		/* debug_spell_errors (text->spell_errors); */
-		if (text->pi)
+		if (text->pi) {
 			for (i =0; i < text->pi->n; i ++)
 				g_print ("item %d offset: %d length: %d\n", i, text->pi->entries [i].item->offset, text->pi->entries [i].item->length);
+				
+			for (i = 0; i < text->text_len; i ++) {
+				PangoLogAttr attr = text->pi->attrs [i];
+				g_print ("log attrs[%d]: %d\n\t", i, (*(int *)&attr) & 0x7ff);
+				if (attr.is_line_break)
+					g_print ("line break, ");
+				if (attr.is_mandatory_break)
+					g_print ("mandatory break, ");
+				if (attr.is_char_break)
+					g_print ("char break, ");
+				if (attr.is_white)
+					g_print ("white, ");
+				g_print ("\n");
+			}
+		}
 	} else if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_TEXTSLAVE) {
 		HTMLTextSlave *slave = HTML_TEXT_SLAVE (obj);
 		gchar *text;
@@ -335,6 +358,7 @@ gtk_html_debug_dump_list_simple (GList *list,
 }
 
 #define D_ATTR_TYPE(x, s) if ((attr = pango_attr_iterator_get (iter, PANGO_ATTR_ ## x))) g_print ("%3d-%3d: %s\n", attr->start_index, attr->end_index, s);
+#define D_ATTR_TYPE_INT(x, s) if ((attr = pango_attr_iterator_get (iter, PANGO_ATTR_ ## x))) { g_print ("%3d-%3d: %s %d\n", attr->start_index, attr->end_index, s, ((PangoAttrInt *)attr)->value); }
 
 void
 gtk_html_debug_list_attrs (PangoAttrList *attrs)
@@ -350,7 +374,7 @@ gtk_html_debug_list_attrs (PangoAttrList *attrs)
 		D_ATTR_TYPE (WEIGHT, "Weight");
 		D_ATTR_TYPE (VARIANT, "Variant");
 		D_ATTR_TYPE (STRETCH, "Stretch");
-		D_ATTR_TYPE (SIZE, "Size");
+		D_ATTR_TYPE_INT (SIZE, "Size");
 		D_ATTR_TYPE (FONT_DESC, "Font Desc");
 		D_ATTR_TYPE (FOREGROUND, "Foreground");
 		D_ATTR_TYPE (BACKGROUND, "Background");
