@@ -44,6 +44,7 @@
 #include "htmlplainpainter.h"
 #include "htmlsearch.h"
 #include "htmlselection.h"
+#include "htmlsettings.h"
 #include "htmltable.h"
 #include "htmltablecell.h"
 #include "htmltext.h"
@@ -666,8 +667,9 @@ pref_right_margin (HTMLPainter *p, HTMLClueFlow *clueflow, HTMLObject *o, gint y
 	  
 	if (clueflow->style == HTML_CLUEFLOW_STYLE_PRE || ! HTML_IS_PLAIN_PAINTER(p))
 		return fixed_margin;
-	
-	return MIN (fixed_margin, 72 * html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL));
+
+	return MIN (fixed_margin, 72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
+					     html_painter_get_e_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL))));
 }
 
 static void
@@ -832,7 +834,8 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	cur = begin;
 
 	old_y = o->y;
-	html_object_calc_size (begin, painter, changed_objs);
+	if (!HTML_IS_TEXT_SLAVE (begin) || HTML_IS_TEXT (begin->prev))
+		html_object_calc_size (begin, painter, changed_objs);
 
 	a = d = height = 0;
 	update_height (begin, html_object_get_valign (begin), &a, &d, &height, &top_align);
@@ -1085,7 +1088,20 @@ get_roman_value (gint value, gboolean lower)
 static gchar *
 get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 {
-	switch (flow->item_type) {
+	HTMLListType type = flow->item_type;
+
+	if (type == HTML_LIST_TYPE_BLOCKQUOTE && flow->levels->len > 0) {
+		int i;
+
+		for (i = flow->levels->len - 1; i >= 0; i --) {
+			if (flow->levels->data [i] != HTML_LIST_TYPE_BLOCKQUOTE) {
+				type = flow->levels->data [i];
+				break;
+			}
+		}
+	}
+
+	switch (type) {
 	case HTML_LIST_TYPE_ORDERED_ARABIC:
 		return g_strdup_printf ("%d. ", flow->item_number);
 	case HTML_LIST_TYPE_ORDERED_LOWER_ALPHA:
@@ -1146,13 +1162,20 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 	int last_indent = 0;
 	gint pixel_size = html_painter_get_pixel_size (painter);
 	gboolean is_plain = HTML_IS_PLAIN_PAINTER (painter);
+	HTMLEngine *e;
+
+	if (painter->widget && GTK_IS_HTML (painter->widget))
+		e = GTK_HTML (painter->widget)->engine;
+	else
+		return;
 	
 	flow = HTML_CLUEFLOW (self);
 
 	for (i = 0; i < flow->levels->len; i++, last_indent = indent) {
 		indent = get_level_indent (flow, i, painter);
 
-		html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLLinkColor)->color);
+		html_painter_set_pen (painter, &html_colorset_get_color_allocated (e->settings->color_set,
+										   painter, HTMLLinkColor)->color);
 		if (is_cite (flow, i)) {
 			if (!is_plain) {
 				area.x = self->x + indent - 5 * pixel_size;
@@ -1201,6 +1224,12 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 	HTMLClueFlow *flow;
 	HTMLObject *first;
 	gchar *marker;
+	HTMLEngine *e;
+
+	if (painter->widget && GTK_IS_HTML (painter->widget))
+		e = GTK_HTML (painter->widget)->engine;
+	else
+		return;
 
 	first = HTML_CLUE (self)->head;
 	if (html_object_is_text (first) && first->next)
@@ -1212,7 +1241,8 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 		html_color_alloc (flow->item_color, painter);
 		html_painter_set_pen (painter, &flow->item_color->color);
 	} else
-		html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
+		html_painter_set_pen (painter, &html_colorset_get_color_allocated (e->settings->color_set,
+										   painter, HTMLTextColor)->color);
 
 	marker = get_item_marker_str (flow, HTML_IS_PLAIN_PAINTER (painter));
 	if (marker) {
@@ -1549,7 +1579,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 		char *start = get_start_tag (self);
 
 		if (start) {
-			if (!save_indent_string (self, state, "<%s>", start))
+			if (!save_indent_string (self, state, "<%s>\n", start))
 				return FALSE;
 		} else {
 			if (!save_indent_string (self, state, ""))
@@ -1580,13 +1610,13 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	}
 
 	if (is_item (self)) {
-		if (next && is_levels_equal (self, next) && !is_item (next)) {
+		if (next && is_levels_equal (self, next) && !is_item (next) && !html_clueflow_contains_table (self)) {
 			if (!html_engine_save_output_string (state, "<BR>\n"))
 				return FALSE;
 		} else if (!html_engine_save_output_string (state, "\n"))
 			return FALSE;
 	} else if (is_levels_equal (self, next) && self->style == next->style) {
-		if (self->style != HTML_CLUEFLOW_STYLE_PRE) {
+		if (self->style != HTML_CLUEFLOW_STYLE_PRE && !html_clueflow_contains_table (self)) {
 			if (!html_engine_save_output_string (state, "<BR>\n"))
 				return FALSE;
 		} else {
@@ -1596,16 +1626,23 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	} else {
 		char *end = get_start_tag (self);
 
-		if (end) {
-			if (!html_engine_save_output_string (state, "</%s>\n", end))
-				return FALSE;
-		} else if (html_clueflow_is_empty (self)) {
-			if (!html_engine_save_output_string (state, "<BR>\n"))
-				return FALSE;
+		if (self->style != HTML_CLUEFLOW_STYLE_PRE) {
+			if ((!html_clueflow_contains_table (self) && !end && next && self->style == next->style) || html_clueflow_is_empty (self)) {
+				if (!html_engine_save_output_string (state, "<BR>\n"))
+					return FALSE;
+			} else {
+				if (!html_engine_save_output_string (state, "\n"))
+					return FALSE;
+			}
 		} else {
 			if (!html_engine_save_output_string (state, "\n"))
 				return FALSE;
 		}
+
+		if (end) {
+			if (!html_engine_save_output_string (state, "</%s>\n", end))
+				return FALSE;
+		}
 	}
 	
 	return TRUE;
@@ -1736,34 +1773,6 @@ append_selection_string (HTMLObject *self,
 	}
 }
 
-static void
-forward_items (PangoItem **items, int *ii, int *io, int n)
-{
-	while (n > 0) {
-		if (items [*ii]->num_chars - *io > n) {
-			*io += n;
-			return;
-		}
-		n -= items [*ii]->num_chars - *io;
-		(*ii) ++;
-		*io = 0;
-	}
-}
-
-static void
-backward_items (PangoItem **items, int *ii, int *io, int n)
-{
-	while (n > 0) {
-		if (*io >= n) {
-			*io -= n;
-			return;
-		}
-		n -= *io;
-		(*ii) --;
-		*io = items [*ii]->num_chars - 1;
-	}
-}
-
 static gboolean
 save_plain (HTMLObject *self,
 	    HTMLEngineSaveState *state,
@@ -1787,7 +1796,8 @@ save_plain (HTMLObject *self,
 	if (HTML_OBJECT_CLASS (&html_clue_class)->save_plain (self, 
 							      buffer_state, 
 							      max_len)) {
-		guchar *s, *space;
+		guchar *s;
+		int offset;
 		
 		if (get_pre_padding (flow, calc_padding (state->engine->painter)) > 0) {
 		        plain_padding (flow, out, FALSE);
@@ -1801,7 +1811,7 @@ save_plain (HTMLObject *self,
 			g_string_append (out, "\n");
 		} else {
 			PangoAttrList *attrs = pango_attr_list_new ();
-			gint bytes = strlen (s), i, slen = g_utf8_strlen (s, -1), clen, n_items, cii, cio;
+			gint bytes = strlen (s), i, slen = g_utf8_strlen (s, -1), clen, n_items;
 			GList *items_list, *cur;
 			PangoContext *pc = gtk_widget_get_pango_context (GTK_WIDGET (state->engine->widget));
 			PangoLogAttr *lattrs;
@@ -1812,19 +1822,38 @@ save_plain (HTMLObject *self,
 			lattrs = g_new (PangoLogAttr, slen + 1);
 			n_items = g_list_length (items_list);
 			items = g_new (PangoItem *, n_items);
-			clen = 0;
-			for (i = 0, cur = items_list; i < n_items; i ++, cur = cur->next) {
-				PangoItem *item;
+			for (i = 0, cur = items_list; i < n_items; i ++, cur = cur->next)
+				items [i] = (PangoItem *) cur->data;
 
-				item = (PangoItem *) cur->data;
-				pango_break (s + item->offset, item->length, &item->analysis, lattrs + clen, item->num_chars + 1);
-				clen += item->num_chars;
-				items [i] = item;
+			offset = 0;
+			for (i = 0; i < n_items; i ++) {
+				PangoItem tmp_item;
+				int start_i, start_offset;
+
+				start_i = i;
+				start_offset = offset;
+				offset += items [i]->num_chars;
+				tmp_item = *items [i];
+				while (i < n_items - 1) {
+					if (tmp_item.analysis.lang_engine == items [i + 1]->analysis.lang_engine) {
+						tmp_item.length += items [i + 1]->length;
+						tmp_item.num_chars += items [i + 1]->num_chars;
+						offset += items [i + 1]->num_chars;
+						i ++;
+					} else
+						break;
+				}
+
+				pango_break (s + tmp_item.offset, tmp_item.length, &tmp_item.analysis, lattrs + start_offset, tmp_item.num_chars + 1);
 			}
+			html_engine_save_buffer_clear_line_breaks (buffer_state, lattrs);
 			g_list_free (items_list);
+			for (i = 0; i < n_items; i ++)
+				pango_item_free (items [i]);
+			g_free (items);
 			pango_attr_list_unref (attrs);
 
-			clen = cii = cio = 0;
+			clen = 0;
 			while (*s) {
 				len = strcspn (s, "\n");
 				len = g_utf8_strlen (s, len);
@@ -1833,43 +1862,31 @@ save_plain (HTMLObject *self,
 				if ((flow->style != HTML_CLUEFLOW_STYLE_PRE) 
 				    && !HTML_IS_TABLE (HTML_CLUE (flow)->head)) {
 					if (len > max_len) {
-						gint l = max_len;
 						gboolean look_backward = TRUE;
-						gint wi, wl, lii, lio, blii, blio;
+						gint wi, wl;
 
 						wl = clen + max_len;
-						lii = cii;
-						lio = cio;
-						forward_items (items, &lii, &lio, max_len);
-						blii = lii;
-						blio = lio;
 
 						if (lattrs [wl].is_white) {
 
-							while (lattrs [wl].is_white && wl < slen) {
+							while (lattrs [wl].is_white && wl < slen)
 								wl ++;
-								forward_items (items, &lii, &lio, 1);
-							}
-							if (wl < slen && html_text_is_line_break (items [lii], lattrs, wl))
-								look_backward = FALSE;
-							else {
-								wl = clen + max_len;
-								lii = blii;
-								lio = blio;
-							}
 
+							if (wl < slen && html_text_is_line_break (lattrs [wl]))
+								look_backward = FALSE;
+							else
+								wl = clen + max_len;
 						}
 
 						if (look_backward) {
 							while (wl > 0) {
-								if (html_text_is_line_break (items [lii], lattrs, wl))
+								if (html_text_is_line_break (lattrs [wl]))
 									break;
 								wl --;
-								backward_items (items, &lii, &lio, 1);
 							}
 						}
 
-						if (wl > clen && wl < slen && html_text_is_line_break (items [lii], lattrs, wl)) {
+						if (wl > clen && wl < slen && html_text_is_line_break (lattrs [wl])) {
 							wi = MIN (wl, clen + max_len);
 							while (wi > 0 && lattrs [wi - 1].is_white)
 								wi --;
@@ -1906,21 +1923,16 @@ save_plain (HTMLObject *self,
 				s += bytes;
 				s = g_utf8_offset_to_pointer (s, skip);
 				clen += len + skip;
-				forward_items (items, &cii, &cio, len + skip);
 
 				if (*s == '\n') {
 					s++;
 					clen ++;
-					forward_items (items, &cii, &cio, 1);
 				}
 			
 				g_string_append_c (out, '\n');
 				firstline = FALSE;
 			}
 			g_free (lattrs);
-			for (i = 0; i < n_items; i ++)
-				pango_item_free (items [i]);
-			g_free (items);
 		}
 
 		if (get_post_padding (flow, calc_padding (state->engine->painter)) > 0) {
@@ -1976,12 +1988,12 @@ get_default_font_style (const HTMLClueFlow *self)
 }
 
 static void
-search_set_info (HTMLObject *cur, HTMLSearch *info, guchar *text, guint pos, guint len)
+search_set_info (HTMLObject *cur, HTMLSearch *info, guchar *text, guint index, guint bytes)
 {
-	guint text_len = 0;
-	guint cur_len;
+	guint text_bytes = 0;
+	guint cur_bytes;
 
-	info->found_len = len;
+	info->found_bytes = bytes;
 
 	if (info->found) {
 		g_list_free (info->found);
@@ -1990,17 +2002,18 @@ search_set_info (HTMLObject *cur, HTMLSearch *info, guchar *text, guint pos, gui
 
 	while (cur) {
 		if (html_object_is_text (cur)) {
-			cur_len = strlen (HTML_TEXT (cur)->text);
-			if (text_len + cur_len > pos) {
+			cur_bytes = HTML_TEXT (cur)->text_bytes;
+			if (text_bytes + cur_bytes > index) {
 				if (!info->found) {
-					info->start_pos = g_utf8_pointer_to_offset (text + text_len,
-										    text + pos);
+					info->start_pos = g_utf8_pointer_to_offset (text + text_bytes,
+										    text + index);
 				}
 				info->found = g_list_append (info->found, cur);
 			}
-			text_len += cur_len;
-			if (text_len >= pos+info->found_len) {
-				info->stop_pos = info->start_pos + info->found_len;
+			text_bytes += cur_bytes;
+			if (text_bytes >= index + info->found_bytes) {
+				info->stop_pos = info->start_pos + g_utf8_pointer_to_offset (text + index,
+											     text + index + info->found_bytes);
 				info->last     = HTML_OBJECT (cur);
 				return;
 			}
@@ -2021,18 +2034,18 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 	HTMLObject *end = cur;
 	HTMLObject *head;
 	guchar *par, *pp;
-	guint text_len;
-	guint eq_len;
-	gint  pos;
+	guint text_bytes;
+	guint eq_bytes;
+	gint index;
 	gboolean retval = FALSE;
 
 	/* printf ("search flow look for \"text\" %s\n", info->text); */
 
-	/* first get flow text_len */
-	text_len = 0;
+	/* first get flow text_bytes */
+	text_bytes = 0;
 	while (cur) {
 		if (html_object_is_text (cur)) {
-			text_len += strlen (HTML_TEXT (cur)->text);
+			text_bytes += HTML_TEXT (cur)->text_bytes;
 			end = cur;
 		} else if (HTML_OBJECT_TYPE (cur) != HTML_TYPE_TEXTSLAVE) {
 			break;
@@ -2040,11 +2053,11 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 		cur = (info->forward) ? cur->next : cur->prev;
 	}
 
-	if (text_len > 0) {
-		par = g_new (gchar, text_len+1);
-		par [text_len] = 0;
+	if (text_bytes > 0) {
+		par = g_new (gchar, text_bytes + 1);
+		par [text_bytes] = 0;
 
-		pp = (info->forward) ? par : par+text_len;
+		pp = (info->forward) ? par : par + text_bytes;
 
 		/* now fill par with text */
 		head = cur = (info->forward) ? *beg : end;
@@ -2052,11 +2065,11 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 		while (cur) {
 			if (html_object_is_text (cur)) {
 				if (!info->forward) {
-					pp -= strlen (HTML_TEXT (cur)->text);
+					pp -= HTML_TEXT (cur)->text_bytes;
 				}
-				strncpy (pp, HTML_TEXT (cur)->text, strlen (HTML_TEXT (cur)->text));
+				strncpy (pp, HTML_TEXT (cur)->text, HTML_TEXT (cur)->text_bytes);
 				if (info->forward) {
-					pp += strlen (HTML_TEXT (cur)->text);
+					pp += HTML_TEXT (cur)->text_bytes;
 				}
 			} else if (HTML_OBJECT_TYPE (cur) != HTML_TYPE_TEXTSLAVE) {
 				break;
@@ -2064,20 +2077,20 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 			cur = (info->forward) ? cur->next : cur->prev;
 		}
 
-		/* set eq_len and pos counters */
-		eq_len = 0;
+		/* set eq_bytes and pos counters */
+		eq_bytes = 0;
 		if (info->found) {
-			pos = info->start_pos + ((info->forward) ? 1 : -1);
+			index = ((guchar *)g_utf8_offset_to_pointer (par, info->start_pos + ((info->forward) ? 1 : -1))) - par;
 		} else {
-			pos = (info->forward) ? 0 : text_len - 1;
+			index = (info->forward) ? 0 : text_bytes - 1;
 		}
 
 		/* FIXME make shorter text instead */
 		if (!info->forward)
-			par [pos+1] = 0;
+			par [index+1] = 0;
 
-		if ((info->forward && pos < text_len)
-		    || (!info->forward && pos>0)) {
+		if ((info->forward && index < text_bytes)
+		    || (!info->forward && index > 0)) {
 			if (info->reb) {
 				/* regex search */
 				gint rv;
@@ -2094,29 +2107,31 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 					p += (info->forward) ? 1 : -1;
 					} */
 
-				while ((info->forward && pos < text_len)
-				       || (!info->forward && pos >= 0)) {
+				while ((info->forward && index < text_bytes)
+				       || (!info->forward && index >= 0)) {
 					rv = regexec (info->reb,
-						      par + pos,
+						      par + index,
 						      1, &match, 0);
 					if (rv == 0) {
 						search_set_info (head, info, par,
-								 pos + match.rm_so, match.rm_eo - match.rm_so);
+								 index + match.rm_so, match.rm_eo - match.rm_so);
 						retval = TRUE;
 						break;
 					}
-					pos += (info->forward) ? 1 : -1;
+					index += (info->forward)
+						? (((guchar *) g_utf8_next_char (par + index)) - par - index)
+						: (((guchar *) g_utf8_prev_char (par + index)) - par - index);
 				}
 #else
-				rv = re_search (info->reb, par, text_len, pos,
-						(info->forward) ? text_len-pos : -pos, NULL);
-				if (rv>=0) {
-					guint found_pos = rv;
-					rv = re_match (info->reb, par, text_len, found_pos, NULL);
+				rv = re_search (info->reb, par, text_bytes, index,
+						(info->forward) ? text_bytes - index : -index, NULL);
+				if (rv >= 0) {
+					guint found_index = rv;
+					rv = re_match (info->reb, par, text_bytes, found_index, NULL);
 					if (rv < 0) {
 						g_warning ("re_match (...) error");
 					}
-					search_set_info (head, info, par, found_pos, rv);
+					search_set_info (head, info, par, found_index, rv);
 					retval = TRUE;
 				} else {
 					if (rv < -1) {
@@ -2127,23 +2142,27 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 			} else {
 				/* substring search - simple one - could be improved
 				   go thru par and look for info->text */
-				while (par [pos]) {
+				while (par [index]) {
 					if (info->trans [(guchar) info->text
-							[(info->forward) ? eq_len : info->text_len - eq_len - 1]]
-					    == info->trans [par [pos]]) {
-						eq_len++;
-						if (eq_len == info->text_len) {
+							[(info->forward) ? eq_bytes : info->text_bytes - eq_bytes - 1]]
+					    == info->trans [par [index]]) {
+						eq_bytes ++;
+						if (eq_bytes == info->text_bytes) {
 							search_set_info (head, info, par,
-									 pos - (info->forward ? eq_len-1 : 0),
-									 info->text_len);
+									 index - (info->forward
+										  ? -(((guchar *) g_utf8_next_char (par + index - eq_bytes)) - par - index)
+										  : 0),
+									 info->text_bytes);
 							retval=TRUE;
 							break;
 						}
 					} else {
-						pos += (info->forward) ? -eq_len : eq_len;
-						eq_len = 0;
+						index += (info->forward) ? -eq_bytes : eq_bytes;
+						eq_bytes = 0;
 					}
-					pos += (info->forward) ? 1 : -1;
+					index += (info->forward)
+						? (((guchar *) g_utf8_next_char (par + index)) - par - index)
+						: (((guchar *) g_utf8_prev_char (par + index)) - par - index);
 				}
 			}
 		}
@@ -2183,7 +2202,7 @@ search (HTMLObject *obj, HTMLSearch *info)
 
 		is_text = html_object_is_text (cur);
 
-		if (html_object_is_text (cur)) {
+		if (is_text) {
 			if (search_text (&cur, info))
 				return TRUE;
 		}
@@ -2898,6 +2917,20 @@ html_clueflow_is_empty (HTMLClueFlow *flow)
 	return FALSE;
 }
 
+gboolean
+html_clueflow_contains_table (HTMLClueFlow *flow)
+{
+	HTMLClue *clue;
+	g_return_val_if_fail (HTML_IS_CLUEFLOW (flow), FALSE);
+
+	clue = HTML_CLUE (flow);
+
+	if (clue->head && HTML_IS_TABLE (clue->head))
+		return TRUE;
+
+	return FALSE;
+}
+
 gint
 html_clueflow_get_line_offset (HTMLClueFlow *flow, HTMLPainter *painter, HTMLObject *child)
 {
