@@ -27,6 +27,8 @@
 
 #include <glib.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdkgc.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtksignal.h>
 
 #include <libgnome/gnome-i18n.h>
@@ -38,6 +40,7 @@
 #include "htmlembedded.h"
 #include "htmlengine.h"
 #include "htmltextslave.h"
+#include "htmlsettings.h"
 #include "gtkhtml-embedded.h"
 #include "gtkhtml.h"
 
@@ -87,7 +90,7 @@ text_width (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *te
 {
 	HTMLTextPangoInfo *pi;
 	GList *glyphs;
-	gint width = 0, n_pi;
+	gint width = 0;
 
 	pi = html_painter_text_itemize_and_prepare_glyphs (HTML_PAINTER (painter), desc, text, bytes, &glyphs, NULL);
 
@@ -188,6 +191,7 @@ alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, Gt
 			      space_asc, space_dsc,
 			      text_width (HTML_GDK_PAINTER (painter), desc, "\xc2\xa0", 2),
 			      text_width (HTML_GDK_PAINTER (painter), desc, "\t", 1),
+			      text_width (HTML_GDK_PAINTER (painter), desc, "e", 1),
 			      text_width (HTML_GDK_PAINTER (painter), desc, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT)),
 			      text_width (HTML_GDK_PAINTER (painter), desc, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE)));
 }
@@ -952,7 +956,8 @@ item_gc (HTMLPainter *p, PangoItem *item, GdkDrawable *drawable, GdkGC *orig_gc,
 	new_gc = gdk_gc_new (drawable);
 	gdk_gc_copy (new_gc, orig_gc);
 	gdk_gc_set_foreground (new_gc,
-			       &html_colorset_get_color_allocated (e->painter, HTMLTextColor)->color);
+			       &html_colorset_get_color_allocated (e->settings->color_set,
+								   e->painter, HTMLTextColor)->color);
 
 	while (tmp_list) {
 		PangoAttribute *attr = tmp_list->data;
@@ -973,7 +978,7 @@ item_gc (HTMLPainter *p, PangoItem *item, GdkDrawable *drawable, GdkGC *orig_gc,
 				gdk_gc_set_foreground (new_gc, &color);
 			else {
 				if (*bg_gc)
-					gdk_gc_unref (*bg_gc);
+					g_object_unref (*bg_gc);
 				*bg_gc = gdk_gc_new (drawable);
 				gdk_gc_copy (*bg_gc, orig_gc);
 				gdk_gc_set_foreground (*bg_gc, &color);
@@ -987,6 +992,8 @@ item_gc (HTMLPainter *p, PangoItem *item, GdkDrawable *drawable, GdkGC *orig_gc,
 		case PANGO_ATTR_STRIKETHROUGH:
 			*strikethrough = TRUE;
 			break;
+		default:
+			break;
 		}
 		tmp_list = tmp_list->next;
 	}
@@ -1040,7 +1047,7 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 		temp_pi = TRUE;
 	}
 	if (pi && pi->n) {
-		GList *gl, *il;
+		GList *gl;
 		guint i, char_offset = 0;
 
 		c_text = text;
@@ -1061,7 +1068,7 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 				pango_glyph_string_extents (str, pi->entries [ii].item->analysis.font, NULL, &log_rect);
 				gdk_draw_rectangle (gdk_painter->pixmap, bg_gc, TRUE, x + width, y - PANGO_PIXELS (PANGO_ASCENT (log_rect)),
 						    PANGO_PIXELS (log_rect.width), PANGO_PIXELS (log_rect.height));
-				gdk_gc_unref (bg_gc);
+				g_object_unref (bg_gc);
 			}
 			gdk_draw_glyphs (gdk_painter->pixmap, gc,
 					 pi->entries [ii].item->analysis.font, x + width, y, str);
@@ -1070,7 +1077,7 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 			else
 				for (i=0; i < str->num_glyphs; i ++)
 					cw += PANGO_PIXELS (str->glyphs [i].geometry.width);
-			gdk_gc_unref (gc);
+			g_object_unref (gc);
 			width += cw;
 			c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
 			char_offset += str->num_glyphs;
