@@ -44,6 +44,7 @@
 #include "htmltext.h"
 #include "htmlrule.h"
 #include "htmltype.h"
+#include "htmlsettings.h"
 
 #include "gtkhtmldebug.h"
 
@@ -133,7 +134,10 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 		} else
 			html_object_move_cursor_before_remove (self, e);
 
-		html_object_change_set   (self,  HTML_CHANGE_ALL_CALC);
+		html_object_change_set (self, HTML_CHANGE_ALL_CALC);
+		html_object_change_set (self->parent, HTML_CHANGE_ALL_CALC);
+		/* force parent redraw */
+		self->parent->width = 0;
 		html_object_remove_child (self->parent, self);
 		*len += html_object_get_recursive_length (self);
 
@@ -145,6 +149,11 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 static gboolean
 merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
+	if (self->parent) {
+		html_object_change_set (self->parent, HTML_CHANGE_ALL_CALC);
+		self->parent->width = 0;
+	}
+
 	return FALSE;
 }
 
@@ -305,10 +314,13 @@ static GdkColor *
 get_bg_color (HTMLObject *o,
 	      HTMLPainter *p)
 {
-	
-	return o->parent
-		? html_object_get_bg_color (o->parent, p)
-		: &((html_colorset_get_color (p->color_set, HTMLBgColor))->color);
+	if (o->parent)
+		return html_object_get_bg_color (o->parent, p);	
+
+	if (p->widget && GTK_IS_HTML (p->widget))
+		return &((html_colorset_get_color (GTK_HTML (p->widget)->engine->settings->color_set,
+						   HTMLBgColor))->color);
+	return NULL;
 }
 
 static HTMLObject*
@@ -533,8 +545,7 @@ save_plain (HTMLObject *self,
 }
 
 static gint
-check_page_split (HTMLObject *self,
-		  gint y)
+check_page_split (HTMLObject *self, HTMLPainter *p, gint y)
 {
 	return 0;
 }
@@ -1210,12 +1221,11 @@ html_object_save_plain (HTMLObject *self,
 }
 
 gint
-html_object_check_page_split  (HTMLObject *self,
-			       gint y)
+html_object_check_page_split  (HTMLObject *self, HTMLPainter *p, gint y)
 {
 	g_return_val_if_fail (self != NULL, 0);
 
-	return (* HO_CLASS (self)->check_page_split) (self, y);
+	return (* HO_CLASS (self)->check_page_split) (self, p, y);
 }
 
 void
@@ -1641,7 +1651,7 @@ object_save_data (GQuark key_id, gpointer data, gpointer user_data)
 	if (!str) {
 		/* printf ("save %s %s -> %s\n", state->save_data_class_name, g_quark_to_string (key_id), (gchar *) data); */
 		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
-						state->save_data_class_name, g_quark_to_string (key_id), data);
+						state->save_data_class_name, g_quark_to_string (key_id), (char *) data);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, g_quark_to_string (key_id), data);
 	}
 }
@@ -1657,12 +1667,12 @@ handle_object_data (gpointer key, gpointer value, gpointer data)
 	if (!str) {
 		/* printf ("clear\n"); */
 		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">-->",
-						state->save_data_class_name, key);
+						state->save_data_class_name, (char *) key);
 		state->data_to_remove = g_slist_prepend (state->data_to_remove, key);
 	} else if (strcmp (value, str)) {
 		/* printf ("change\n"); */
 		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
-						state->save_data_class_name, key, str);
+						state->save_data_class_name, (char *) key, str);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, key, value);
 	}
 }
